#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <portbits.h>
#include "8x12_horizontal_MSB_1.h"
#include "global.h"
#include "lcd.h"

unsigned char * const mem=(unsigned char * const)DDRAM;
unsigned char cgram[16][8];

void lcd_init(void)
{	cli();
	wdt_disable();
	MCUCR=(1<<SRE)|(0<<SRW10);					// Enable XMEM		!!! Die Null vor dem SRW Bit durch eine 1 ersetzen, !!!
												//					!!! wenn der verwendete SRAM mehr als 35ns hat 		!!!
	OCR1A=10;
	OCR1B=128;
	TCCR1A=(1<<COM1B1)|(1<<COM1B0)|(1<<WGM10);	// Enable PWM (Kontrast)
	TCCR1B=1;									// 8bit PWM

	OCR0=RELOAD;
	TCCR0=(1<<WGM01)|2;							// 2MHz Timer Takt, CTC
	TIMSK=(1<<OCIE0);

	sei();
	lcd_clear();

	lcd_block(50,10,269,79,0,255);
	lcd_block(60,20,109,69,64,255);
	lcd_block(150,15,189,45,64,64);
	lcd_block(180,40,219,75,128,128);
	lcd_block(200,20,239,50,255,255);
	lcd_string(10,120,PSTR("320x240 LCD Controller"),255,0);
	lcd_string(15,140,PSTR("\xB8"" by Benedikt"),255,0);

	wdt_enable(WDTO_250MS);
	ENABLE_VLCD=1;	
}

void lcd_string(unsigned char x, unsigned char y, char *txt, unsigned char tcol, unsigned char bcol)
{	unsigned char c;
	for(;;)
	{	c=pgm_read_byte(txt++);
		if (!c)
			break;
		lcd_writechar(x++,y,c,tcol,bcol);
	}
}

void lcd_clear(void)
{	unsigned short i;
	for (i=0; i<(65536-DDRAM); i++)
		mem[i]=0;
}

void bresenham_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{	signed short dx,dy,x,y;
	signed char sdx,sdy;
	unsigned short i,px,py;
	dx=(signed int)x2-x1;      // the horizontal distance of the line
	dy=(signed int)y2-y1;      // the vertical distance of the line
	if (dx==0)
		sdx=0;
	if (dx>0)
		sdx=1;
	else
	{  	sdx=-1;
		dx=-dx;
	}
	if (dy==0)
		sdy=0;
	if (dy>0)
		sdy=1;
	else
	{	sdy=-1;
		dy=-dy;
	}
	x=dx*2;
	y=dy*2;
	px=x1;
	py=y1;
	lcd_setpixel (px,py,color);

	if (dx>=dy)					 // the line is more horizontal than vertical
	{	for(i=0;i<dx;i++)
		{	y+=dy;
			if (y>=dx)
			{	y-=dx;
				py+=sdy;
			}
			px+=sdx;
			lcd_setpixel (px,py,color);
		}
	}
	else				 		// the line is more vertical than horizontal
	{	for(i=0;i<dy;i++)
		{	x+=dx;
			if (x>=dy)
			{	x-=dy;
				px+=sdx;
			}
			py+=sdy;
			lcd_setpixel (px,py,color);
		}
	}
}

void lcd_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{   unsigned short temp;
	if (y1==y2)
	{	if (x1>x2)
		{	temp=x1;
			x1=x2;
			x2=temp;
		}
		for (temp=x1; temp<=x2; temp++)
			lcd_setpixel (temp,y1,color);

	}
	else if (x1==x2)
	{	if (y1>y2)
		{	temp=y1;
			y1=y2;
			y2=temp;
		}
		for (temp=y1; temp<=y2; temp++)
			lcd_setpixel (x1,temp,color);
	}
	else
		bresenham_line (x1, y1, x2, y2, color);
}

void lcd_block (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color1, unsigned char color2)
{	unsigned short y;
	if (x2>x1+1)
	{	for (y=y1+1; y<y2; y++)
			lcd_line(x1+1,y,x2-1,y,color1);
	}
	lcd_line(x1,y1,x2,y1,color2);
	lcd_line(x2,y1,x2,y2,color2);
	lcd_line(x2,y2,x1,y2,color2);
	lcd_line(x1,y2,x1,y1,color2);
}

void lcd_writecgram(unsigned char x, unsigned char y, unsigned char nr,unsigned char textcol,unsigned char backcol)
{	nr&=15;
	lcd_writebyteex(x,y+0,cgram[nr][0],textcol, backcol);
	lcd_writebyteex(x,y+1,cgram[nr][0],textcol, backcol);
	lcd_writebyteex(x,y+2,cgram[nr][1],textcol, backcol);
	lcd_writebyteex(x,y+3,cgram[nr][2],textcol, backcol);
	lcd_writebyteex(x,y+4,cgram[nr][2],textcol, backcol);
	lcd_writebyteex(x,y+5,cgram[nr][3],textcol, backcol);
	lcd_writebyteex(x,y+6,cgram[nr][4],textcol, backcol);
	lcd_writebyteex(x,y+7,cgram[nr][5],textcol, backcol);
	lcd_writebyteex(x,y+8,cgram[nr][5],textcol, backcol);
	lcd_writebyteex(x,y+9,cgram[nr][6],textcol, backcol);
	lcd_writebyteex(x,y+10,cgram[nr][7],textcol, backcol);
	lcd_writebyteex(x,y+11,cgram[nr][7],textcol, backcol);
}
